#define	F_CPU	3686400
#define sbi(ADDRESS,BIT) ((ADDRESS) |= (1<<(BIT)))
#define cbi(ADDRESS,BIT) ((ADDRESS) &= ~(1<<(BIT)))

#include <avr\io.h>
#include <util\delay.h>

void wait_ms( int milliSec )
{
	_delay_loop_2( 1 * ( F_CPU / ( 1000 / 4 ) ) * milliSec );
}

void wait_us( int microSec )
{
	_delay_loop_2( 1 * ( F_CPU / ( 1000000 / 4 ) ) * microSec );
}


void lcd_send( char data )
{
	char rs = PORTD;
	rs &= 4;
	char tmp = data;
	tmp &= 0xf0;
	tmp |= rs;
	PORTD = tmp;
	sbi( PORTD, 3 );
	cbi( PORTD, 3 );
	tmp = data;
	tmp &= 0x0f;
	tmp *= 16;
	tmp |= rs;
	PORTD = tmp;
	sbi( PORTD, 3 );
	cbi( PORTD, 3 );
	wait_ms( 1 );
}

void lcd_cmd( char cmd )
{
	cbi( PORTD, 2 );
	lcd_send( cmd );
}

void lcd_clear()
{
	lcd_cmd( 0x01 );
	wait_ms( 2 );
}

void lcd_home()
{
	lcd_cmd( 0x02 );
	wait_ms( 2 );
}

void lcd_on()
{
	lcd_cmd( 0x0E );
}

void lcd_off()
{
	lcd_cmd( 0x08 );
}

void lcd_goto( int row, int col )
{
    uint8_t tmp = 0x00;

    switch (row) 
    {
        case 1: tmp=0x80+0x00+col; break;    // 1. Zeile
        case 2: tmp=0x80+0x40+col; break;    // 2. Zeile
        case 3: tmp=0x80+0x10+col; break;    // 3. Zeile
        case 4: tmp=0x80+0x50+col; break;    // 4. Zeile
    }
	lcd_cmd( tmp );
}

void lcd_init()
{
	DDRD = 0x0ff;
	PORTD = 0;
	wait_ms( 50 );
	PORTD = 0x20;
	sbi( PORTD, 3 );
	cbi( PORTD, 3 );
	wait_ms( 5 );
	lcd_cmd( 0x28 );
	lcd_off();
	lcd_clear();
	lcd_cmd( 0x06 );
	lcd_on();
}

void lcd_write( char text )
{
	sbi( PORTD, 2 );
	lcd_send( text );
}

void lcd_writeText( char* pText )
{
	while ( pText[0] != 0 )
	{
		lcd_write( pText[0] );
		pText++;
	}
}

